# This proxy will keep postgres and orientdb syncronized
#   When some client call UserRepositoryProxy, it will delegate the missing methods for ActiveRecord User model
class UserRepositoryProxy

  def initialize(orientdb_user_repository)
    @orientdb_user_repository = orientdb_user_repository
  end

  def create_graph_node(user)
    if user.persisted?
      if orientdb_user_repository.create_graph_node user
        update_user_refs user
      end
    else
      raise_not_persisted_error
    end
  end

  def get_graph_id(user)
    orientdb_user_repository.get_graph_id user
  end

  def destroy_graph_node(user)
    orientdb_user_repository.destroy_graph_node user
  end

  def save(user)
    user.save
  end

  def save!(user)
    user.save!
  end

  def method_missing(method_name, *arguments, &block)
    User.send(method_name, *arguments, &block)
  end

  def respond_to?(method_name, include_private = false)
    User.respond_to?(method_name) || super
  end


  private

  attr_reader :orientdb_user_repository

  def update_user_refs(user)
    rid = get_graph_id user
    user.rid = rid
    user.save!
  end

  def check_references(user)
    rid = get_graph_id user
    (rid == user.rid)
  end

  def raise_not_persisted_error
    raise NotPersistedRecordError, 'The user wasn`t` persisted yet.'
  end

end
