class ThumbnailGeneratorWorker

  include Sidekiq::Worker
  include RepositoriesProxy
  include Thumbnail::Creation
  include Thumbnail::Formats

  def perform(learning_object_id)

    item = learning_object_repository.find(learning_object_id)
    filename = item.get_bitstream_filename_of "ORIGINAL"

    size = "530x298"

    unless accepted_formats.include? file_format filename
      item.thumbnail = default_thumbnail
    else
      begin
        retrieve_link = item.get_bitstream_retrievelink_of "ORIGINAL"
        file = download_bitstream(retrieve_link,filename)
      rescue
        puts "ERROR!!! Some error occurred during file download."
      else
        item.thumbnail = generate_thumbnail(file,filename,size)
        delete_downloaded_bitstream(file)
      end
    end

    learning_object_repository.update_property(item,"thumbnail",item.thumbnail)

  end

  private

  def delete_downloaded_bitstream(file)
    File.unlink(file) if File.exist?(file)
  end

  def download_bitstream(url, filename)
    output_dir = "/tmp"

    output_file = "#{output_dir}/#{filename}"

    c = Curl::Easy.new(url)
    c.ssl_verify_peer = false
    c.ssl_verify_host = false
    File.open(output_file, 'wb') do |f|
      # c.on_progress { |dl_total, dl_now, ul_total, ul_now|
      #   puts "#{dl_total}, #{dl_now}, #{ul_total}, #{ul_now}"; true
      # }
      c.on_body {|data| f << data; data.size }
      c.perform
    end

    return output_file
  end

end
